function DisplayBlock(BlockContent,DisplayController,ConditionalController){
    var self = this; self.name = "DisplayBlock"; // MANDATORY

    this.data = BlockContent;
    this.renderer = DisplayController;
    this.conditional = ConditionalController;
    
    this.loaded = false;
    this.preloaded = false;
    this.rendering = false;
    this.reference = false;
    
    this.layer = false;
    this.rectangle = false;
    this.remaining = 0;
    this.finish = false;
    
    this.next = false;
    this.last = false;
     
    this.scene = false;
    this.rendering_scene = false;
    
    if(self.data){
        this.reference = self.data.getId();
    }
       
    this.load = function(finish,layer,rectangle){
        if(typeof(finish) === "undefined"){finish = false;}
        self.log("load");
        if(self.data){
            self.data.position = -1;
            if(!self.loaded){
                self.finish = finish;
                self.layer = layer;
                self.rectangle = rectangle;
                self.data.build(1);
                self.lookForNext();
                if(self.finish){
                    self.scene = self.renderer.prepare_scene(self.next.template,self.next.content,true,self.layer);
                }else{
                    if(self.renderer.support_multi_video){
                      self.renderer.preload(self.next.content,self.layer,true,self.rectangle);
                    }
                }
                self.loaded = true;
            }        
            self.remaining = self.data.getTotalPlays();
        }
    };
    
    this.render = function(layer,rectangle,finish){
        if(!self.loaded){
            self.load(finish,layer,rectangle);
        }
        
        if(!self.rendering){         
            if(!self.renderer.checkAvailability(self.next.content,self.layer)){
                self.rendering = false;
            }else{
                var capsule = self.renderer.getDisplayContentElement(self.next.content,self.layer);
                $(capsule).one('DisplayContentElementPlay',self.onBlockDisplayContentElementStart);
                $(capsule).one('DisplayContentElementFinish',self.onBlockDisplayContentElementFinish);
                if(self.finish){
                    self.rendering_scene = self.renderer.render_scene(self.scene,self.rendering_scene);
                }else{
                    self.renderer.render(self.next.content,self.layer,true,self.rectangle);     
                }
                self.rendering = true;
            }    
        }
    };
    
    this.lookForNext = function(){
        var found = false;
        var count_block_items = self.data.getNumberOfElements();
        while(!found && count_block_items > 0){
            self.next = self.data.updateSlot(0,self.conditional);
            if(!self.next){
                self.log("NO block reference");
            }else{
                var content_element = self.data.getContentElement(self.next.content);
                id = self.renderer.addDisplayElement(content_element,self.layer);
                if(self.renderer.checkAvailability(id,self.layer)){
                    if(content_element.verify(self.getDate(true).getTime())){
                      if(self.verify_schedule(self.next)){
                        found = true;
                      }else{
                        self.log("Block item out of schedule ..."); 
                      }
                    }else{
                        self.log("Block item expired or not active yet ..."); 
                    }
                }else{
                    self.log("Block item not avaiable");    
                }
            }               
            count_block_items--;
        } 
    };
        
    this.onBlockDisplayContentElementFinish = function(e){    
        e.stopImmediatePropagation();
        $(e.currentTarget).off('DisplayContentElementPlay');        
        $(e.currentTarget).off('DisplayContentElementFinish');        
        // UNLOAD TARGET !
        
        if(!self.finish || (self.finish && self.remaining > 1)){
            var capsule = self.renderer.getDisplayContentElement(self.next.content,self.layer);
            $(capsule).one('DisplayContentElementPlay',self.onBlockDisplayContentElementStart);
            $(capsule).one('DisplayContentElementFinish',self.onBlockDisplayContentElementFinish);
            if(self.finish){
                self.rendering_scene = self.renderer.render_scene(self.scene,self.rendering_scene);
            }else{
                self.renderer.render(self.next.content,self.layer,true,self.rectangle);     
            }
            if(self.finish){
                self.remaining--;
            }
        }else{
            self.log("Send block is finished");
            $(self).trigger("DisplayBlockFinished");
        }   
    };
    
    
    this.onBlockDisplayContentElementStart = function(e){
        self.last = self.next;
        self.lookForNext();
        self.renderer.checkAvailability(self.next.content,self.layer);  
        setTimeout(function(){
            if(self.finish){
                self.scene = self.renderer.prepare_scene(self.next.template,self.next.content,true,self.layer);
            }else{
              if(self.renderer.support_multi_video){
                self.renderer.preload(self.next.content,self.layer,true,self.rectangle);
              }
            }
        },500);
    };

    this.unload = function(){ 
        self.loaded = false;
        if(typeof(self.data) !== "undefined"){
          if(typeof(self.data.contents) !== "undefined"){
            for(var content in self.data.contents){
              try{
                self.renderer.cancel_all(content,self.layer,true);    
              }catch(e){
              }
            }
          }
        }
        self.rendering = false;
    };
    
    this.destroy = function(){
        // Completely unload
    };
    
    this.verify_schedule = function(block){
        var start = 0;
        var end = 0;
        var content_block_item =false;
        if(block){
            content_block_item = $(self.data.data[0]).find("content_block_item[position='"+block.position+"']");
            start = parseInt($(content_block_item).attr('startHour'));
            end = parseInt($(content_block_item).attr('endHour'));
        }
        if(end == 0){end = 1440;}

        if(isNaN(start) && isNaN(end)){ return true; }
        var date = self.getDate();
        var hours = 0;
        var minutes = 0;
        if(self.server_time_activated){
            hours = date.getUTCHours();
            minutes = date.getUTCMinutes();
        }else{
            hours = date.getHours();
            minutes = date.getMinutes();
        }

        var total_minutes = (hours*60) + minutes;

        self.log("Schedule check : "+start+" < [ "+total_minutes+" ] > "+end);

        if(start <= total_minutes && end >= total_minutes){
            hours = null;
            minutes = null;
            date = null;
            start = null;
            end = null;
            return true;
        }else if(start >= end && (total_minutes <= end || total_minutes >= start)){
          hours = null;
          minutes = null;
          date = null;
          start = null;
          end = null;
          return true;
        }
        return false;
    };
}
DisplayBlock.prototype = new SystemController();
